#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>
#include <PubSubClient.h>
#include <EEPROM.h>
#include <ESP8266HTTPUpdateServer.h>

extern "C" {
#include "user_interface.h"
}

#define Taster 0
#define Relais 12
#define LED 13          // grüne LED
boolean val = 0;        // SchaltZustand

#define DNS             // Hier wird "captive portal"(beliebige Domen führt zur Startseite) eingeschaltet
IPAddress apIP(192, 168, 168, 30);

#if defined DNS
#include <DNSServer.h>
const byte DNS_PORT = 53;
DNSServer dnsServer;
#endif

char ssid[32] = "fhgdhfghd\0";         // ssid und passwort werden aus EEPROM später eingelesen
char passwort[64] = "gghdfgh\0";       // hier nur das Platzhalter
String konfig[7];
#define BUFFER_SIZE 100
unsigned int MinutenAlt;

ESP8266WebServer server;
EspClass esp;
int z = 0;                   // Aktuelle EEPROM-Adresse zum lesen
#include "Setup2.h"          // Unterprogramme für Setup 
unsigned int timer = 0;

WiFiClient wclient;
PubSubClient client(wclient);

void callback(const MQTT::Publish& pub)      // Diese funktion wird bei empfang einer subscribe Publish ausgeführt
{
  String Empfangen = pub.payload_string();   // Auswertung der empfanener Nachrichts
  if (Empfangen == "0")
  {
    val = 0;                                 // Ausschalten
    timer = 0;                               // Falls Timer leuft Timer deaktivieren
  }

  else if (Empfangen == "1") val = 1;          // Einschalten
  else if (Empfangen == "2") val = !val;       // Umschalten
  else
  { // andere Zahl oder Text
    if (Empfangen.toInt() != 0)   // wenn Zahl enthalten
    {
      timer = Empfangen.toInt();  // Timer setzen
      val = 1;                    // und einschalten
    }
  }
  digitalWrite(Relais, val);
  client.publish(MQTT::Publish(konfig[6], String(val)).set_qos(0));  // SchaltStatus  melden
}



void setup()                // Wird 1 Mal beim Start ausgefuehrt
{
  pinMode(Taster, INPUT_PULLUP);
  pinMode(LED, OUTPUT);
  pinMode(Relais, OUTPUT);
  EEPROM.begin(512);

  //------------------------------------ Eingang in Setupmodus nechste 2 Sekunden möglich --------------

  for (int i = 0; i < 20; i++)
  {
    digitalWrite(LED, 0);
    delay(50);
    digitalWrite(LED, 1);
    delay(50);
    if (!digitalRead(Taster)) // Taster wurde betätigt
    {
      digitalWrite(LED, 0);   // Dabei leuchtet LED permanent
      Einstellen();
    }
  }
  //----------------------------------- Normale Start -----------------------------------------------

  z = 0;
  LeseEeprom(ssid, sizeof(ssid));        // EEPROM lesen
  if (ssid[0] == 255) // Wenn EEPROM leer ist dann:
  {
    for (int i = 0; i < 20; i++) EEPROM.write(i, '\0');
    EEPROM.commit();

    digitalWrite(LED, 0);
    Einstellen();
  }
  LeseEeprom(passwort, sizeof(passwort));
  for (int k = 0; k < 7; k++) LeseEepromStr(&konfig[k], 60);

  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, passwort);
  WiFi.hostname("MQTT-Relais");
  int timout = 0;
  while (WiFi.status() != WL_CONNECTED)
  {
    timout++;
    if  (timout > 60) // Wenn Anmeldung nicht möglich
    {
      digitalWrite(LED, 0);    // LED ON für 5 Sekunden
      delay(5000);
      digitalWrite(LED, 1);
      delay(100);
      break;
    }        // End timeout
    delay(100);
    digitalWrite(LED, 0);            // Beim WLAN Anmeldung blinken
    delay(100);
    digitalWrite(LED, 1);
    delay(100);
  }                         // Verbunden

  Serial.begin(115200);
     if (!client.connected()) //------------------------- MQTT nicht verbunden ----------------------------------------
    {
      client.set_server(konfig[0], konfig[1].toInt());
      if (client.connect(MQTT::Connect(konfig[4]).set_auth(konfig[2], konfig[3])))
      {
        client.publish(MQTT::Publish("Hallo", konfig[4]).set_qos(0));
        client.set_callback(callback);
        client.subscribe(konfig[5]);
      }
    } //----------------------------------------------- ende MQTT nicht verbunden -------------------------------------
}


void loop()
{
  // --------------------------------- Taster ----------------------------------------------------------
  if (!digitalRead(Taster)) // Wenn Taster betätigt
  {
    delay(50);              // Entprellung
    if (!digitalRead(Taster))  //immer noch betätigt? Noch mal prüfen
    {
      val = !val;           // Umschalten
      digitalWrite(Relais, val);
       if (client.connected()) client.publish(MQTT::Publish(konfig[6], String(val)).set_qos(0));
      while (!digitalRead(Taster))
      {
       if (client.connected())client.loop(); // Warten bis der Taster losgelassen wird
      }
    }
  }// Ende Taster betätigt


  // --------------------- Timer ------------------------------------------------
  unsigned int Minuten;
  Minuten = millis() / 60000;
  if (Minuten != MinutenAlt)  // Passirt jede Minute
  {
    if (timer > 0)  // Wenn timer gesetzt
    {
      timer--;       // Timer rükwärts zählen
      if (timer == 0) // Wenn timer abgelaufen
      {
        val = 0;      // Relais ausschalten
        digitalWrite(Relais, val);
        if (client.connected()) client.publish(MQTT::Publish(konfig[6], String(val)).set_qos(0));
      } // ende  Wenn timer abgelaufen
    } // ende  Wenn timer gesetzt
    MinutenAlt = Minuten;
    
       if (!client.connected()) //------------------------- MQTT nicht verbunden ----------------------------------------
    {
      client.set_server(konfig[0], konfig[1].toInt());
      if (client.connect(MQTT::Connect(konfig[4]).set_auth(konfig[2], konfig[3])))
      {
        client.publish(MQTT::Publish("Hallo", konfig[4]).set_qos(0));
        client.set_callback(callback);
        client.subscribe(konfig[5]);
      }
    } //----------------------------------------------- ende MQTT nicht verbunden -------------------------------------
  }                      // Ende Passirt jede Minute

  if (WiFi.status() == WL_CONNECTED)
  {
    if (client.connected())  // MQTT  verbunden
    {
      client.loop();
    }// ende mit MQTT verbunden
  } // ende mit WLAN verbunden
}
